@@ -3,7 +3,7 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006-2008, 2011-2014 Lukáš Lalinský
-# Copyright (C) 2009, 2018-2021 Philipp Wolfer
+# Copyright (C) 2009, 2018-2022 Philipp Wolfer
 # Copyright (C) 2012 Chad Wilson
 # Copyright (C) 2012-2013 Michael Wiencek
 # Copyright (C) 2013-2021 Laurent Monin
@@ -42,7 +42,7 @@ PICARD_APP_NAME = "Picard"
 PICARD_DISPLAY_NAME = "MusicBrainz Picard"
 PICARD_APP_ID = "org.musicbrainz.Picard"
 PICARD_DESKTOP_NAME = PICARD_APP_ID + ".desktop"
-PICARD_VERSION = Version(2, 7, 3, 'final', 0)
+PICARD_VERSION = Version(2, 8, 3, 'final', 0)
 
 
 # optional build version
@@ -85,6 +85,7 @@ api_versions = [
     "2.5",
     "2.6",
     "2.7",
+    "2.8",
 ]
 
 api_versions_tuple = [Version.from_string(v) for v in api_versions]
@@ -124,9 +125,9 @@ def crash_handler():
     if not app:
         app = QApplication(sys.argv)
     msgbox = QMessageBox()
-    msgbox.setIcon(QMessageBox.Critical)
+    msgbox.setIcon(QMessageBox.Icon.Critical)
     msgbox.setWindowTitle("Picard terminated unexpectedly")
-    msgbox.setTextFormat(Qt.RichText)
+    msgbox.setTextFormat(Qt.TextFormat.RichText)
     msgbox.setText(
         'An unexpected error has caused Picard to crash. '
         'Please report this issue on the <a href="https://tickets.metabrainz.org/projects/PICARD">MusicBrainz bug tracker</a>.')
@@ -136,7 +137,7 @@ def crash_handler():
             'A logfile has been written to <a href="{0}">{1}</a>.'
             .format(logfile_url.url(), logfile))
     msgbox.setDetailedText(trace)
-    msgbox.setStandardButtons(QMessageBox.Close)
-    msgbox.setDefaultButton(QMessageBox.Close)
+    msgbox.setStandardButtons(QMessageBox.StandardButton.Close)
+    msgbox.setDefaultButton(QMessageBox.StandardButton.Close)
     msgbox.exec_()
     app.quit()
