@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2007 Oliver Charles
 # Copyright (C) 2007, 2010-2011 Lukáš Lalinský
-# Copyright (C) 2007-2011, 2014, 2018-2021 Philipp Wolfer
+# Copyright (C) 2007-2011, 2014, 2018-2022 Philipp Wolfer
 # Copyright (C) 2011 Michael Wiencek
 # Copyright (C) 2011-2012, 2015 Wieland Hoffmann
 # Copyright (C) 2013-2015, 2018-2021 Laurent Monin
@@ -43,6 +43,7 @@ from PyQt5.QtCore import (
 from picard import log
 from picard.config import get_config
 from picard.const import DEFAULT_COVER_IMAGE_FILENAME
+from picard.const.sys import IS_WIN
 from picard.coverart.utils import (
     Id3ImageType,
     image_type_as_id3_num,
@@ -54,12 +55,14 @@ from picard.util import (
     encode_filename,
     imageinfo,
     is_absolute_path,
+    periodictouch,
+    sanitize_filename,
 )
 from picard.util.scripttofilename import script_to_filename
 
 
 _datafiles = dict()
-_datafile_mutex = QMutex(QMutex.Recursive)
+_datafile_mutex = QMutex(QMutex.RecursionMode.Recursive)
 
 
 class DataHash:
@@ -77,6 +80,7 @@ class DataHash:
                 with os.fdopen(fd, "wb") as imagefile:
                     imagefile.write(data)
                 _datafiles[self._hash] = self._filename
+                periodictouch.register_file(self._filename)
                 log.debug("Saving image data %s to %r" % (self._hash, self._filename))
             else:
                 self._filename = _datafiles[self._hash]
@@ -93,6 +97,7 @@ class DataHash:
         if self._filename:
             try:
                 os.unlink(self._filename)
+                periodictouch.unregister_file(self._filename)
             except BaseException:
                 pass
             else:
@@ -170,7 +175,7 @@ class CoverArtImage:
         self.url = QUrl(url)
         self.host = self.url.host()
         self.port = self.url.port(443 if self.url.scheme() == 'https' else 80)
-        self.path = self.url.path(QUrl.FullyEncoded)
+        self.path = self.url.path(QUrl.ComponentFormattingOption.FullyEncoded)
         if self.url.hasQuery():
             query = QUrlQuery(self.url.query())
             self.queryargs = dict(query.queryItems())
@@ -328,7 +333,8 @@ class CoverArtImage:
             return
         config = get_config()
         if config.setting["image_type_as_filename"] and not self.is_front_image():
-            filename = self.maintype
+            win_compat = IS_WIN or config.setting["windows_compatibility"]
+            filename = sanitize_filename(self.maintype, win_compat=win_compat)
             log.debug("Make cover filename from types: %r -> %r",
                       self.types, filename)
         else:
