@@ -5,10 +5,10 @@
 # Copyright (C) 2006 Lukáš Lalinský
 # Copyright (C) 2011-2014 Michael Wiencek
 # Copyright (C) 2012 Frederik “Freso” S. Olesen
-# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2015, 2018-2021 Laurent Monin
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Suhas
-# Copyright (C) 2018-2019 Philipp Wolfer
+# Copyright (C) 2018-2022 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,7 +25,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from locale import strxfrm
 from operator import itemgetter
 
 from PyQt5 import (
@@ -33,7 +32,10 @@ from PyQt5 import (
     QtWidgets,
 )
 
-from picard import config
+from picard.config import (
+    ListOption,
+    get_config,
+)
 from picard.const import (
     RELEASE_COUNTRIES,
     RELEASE_FORMATS,
@@ -41,7 +43,7 @@ from picard.const import (
     RELEASE_SECONDARY_GROUPS,
 )
 from picard.const.sys import IS_WIN
-from picard.i18n import gettext_attr
+from picard.util import strxfrm
 
 from picard.ui.options import (
     OptionsPage,
@@ -70,7 +72,7 @@ class TipSlider(ClickableSlider):
         self.opt = QtWidgets.QStyleOptionSlider()
         self.setMinimum(self._minimum)
         self.setMaximum(self._maximum)
-        self.setOrientation(QtCore.Qt.Horizontal)
+        self.setOrientation(QtCore.Qt.Orientation.Horizontal)
         self.setSingleStep(self._step)
         self.setTickInterval(self._step)
         self.setPageStep(self._pagestep)
@@ -156,11 +158,12 @@ class ReleasesOptionsPage(OptionsPage):
     PARENT = "metadata"
     SORT_ORDER = 10
     ACTIVE = True
+    HELP_URL = '/config/options_releases.html'
 
     options = [
-        config.ListOption("setting", "release_type_scores", _release_type_scores),
-        config.ListOption("setting", "preferred_release_countries", []),
-        config.ListOption("setting", "preferred_release_formats", []),
+        ListOption("setting", "release_type_scores", _release_type_scores),
+        ListOption("setting", "preferred_release_countries", []),
+        ListOption("setting", "preferred_release_formats", []),
     ]
 
     def __init__(self, parent=None):
@@ -171,12 +174,12 @@ class ReleasesOptionsPage(OptionsPage):
         self._release_type_sliders = {}
 
         def add_slider(name, griditer, context):
-            label = gettext_attr(name, context)
-            self._release_type_sliders[name] = \
-                ReleaseTypeScore(self.ui.type_group,
-                                 self.ui.gridLayout,
-                                 label,
-                                 next(griditer))
+            label = pgettext_attributes(context, name)
+            self._release_type_sliders[name] = ReleaseTypeScore(
+                self.ui.type_group,
+                self.ui.gridLayout,
+                label,
+                next(griditer))
 
         griditer = RowColIter(len(RELEASE_PRIMARY_GROUPS)
                               + len(RELEASE_SECONDARY_GROUPS)
@@ -184,28 +187,37 @@ class ReleasesOptionsPage(OptionsPage):
         for name in RELEASE_PRIMARY_GROUPS:
             add_slider(name, griditer, context='release_group_primary_type')
         for name in sorted(RELEASE_SECONDARY_GROUPS,
-                           key=lambda v: gettext_attr(v, 'release_group_secondary_type')):
+                           key=lambda v: pgettext_attributes('release_group_secondary_type', v)):
             add_slider(name, griditer, context='release_group_secondary_type')
 
-        self.reset_preferred_types_btn = QtWidgets.QPushButton(self.ui.type_group)
-        self.reset_preferred_types_btn.setText(_("Reset all"))
-        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
+        reset_types_btn = QtWidgets.QPushButton(self.ui.type_group)
+        reset_types_btn.setText(_("Reset all"))
+        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
         sizePolicy.setHorizontalStretch(0)
         sizePolicy.setVerticalStretch(0)
-        sizePolicy.setHeightForWidth(self.reset_preferred_types_btn.sizePolicy().hasHeightForWidth())
-        self.reset_preferred_types_btn.setSizePolicy(sizePolicy)
+        sizePolicy.setHeightForWidth(reset_types_btn.sizePolicy().hasHeightForWidth())
+        reset_types_btn.setSizePolicy(sizePolicy)
         r, c = next(griditer)
-        self.ui.gridLayout.addWidget(self.reset_preferred_types_btn, r, c, 1, 2)
-        self.reset_preferred_types_btn.clicked.connect(self.reset_preferred_types)
+        self.ui.gridLayout.addWidget(reset_types_btn, r, c, 1, 2)
+        reset_types_btn.clicked.connect(self.reset_preferred_types)
+
+        self.setTabOrder(reset_types_btn, self.ui.country_list)
+        self.setTabOrder(self.ui.country_list, self.ui.preferred_country_list)
+        self.setTabOrder(self.ui.preferred_country_list, self.ui.add_countries)
+        self.setTabOrder(self.ui.add_countries, self.ui.remove_countries)
+        self.setTabOrder(self.ui.remove_countries, self.ui.format_list)
+        self.setTabOrder(self.ui.format_list, self.ui.preferred_format_list)
+        self.setTabOrder(self.ui.preferred_format_list, self.ui.add_formats)
+        self.setTabOrder(self.ui.add_formats, self.ui.remove_formats)
 
         self.ui.add_countries.clicked.connect(self.add_preferred_countries)
         self.ui.remove_countries.clicked.connect(self.remove_preferred_countries)
         self.ui.add_formats.clicked.connect(self.add_preferred_formats)
         self.ui.remove_formats.clicked.connect(self.remove_preferred_formats)
-        self.ui.country_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
-        self.ui.preferred_country_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
-        self.ui.format_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
-        self.ui.preferred_format_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
+        self.ui.country_list.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
+        self.ui.preferred_country_list.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
+        self.ui.format_list.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
+        self.ui.preferred_format_list.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
 
     def restore_defaults(self):
         # Clear lists
@@ -216,6 +228,7 @@ class ReleasesOptionsPage(OptionsPage):
         super().restore_defaults()
 
     def load(self):
+        config = get_config()
         scores = dict(config.setting["release_type_scores"])
         for (release_type, release_type_slider) in self._release_type_sliders.items():
             release_type_slider.setValue(scores.get(release_type,
@@ -227,6 +240,7 @@ class ReleasesOptionsPage(OptionsPage):
                               self.ui.format_list, self.ui.preferred_format_list)
 
     def save(self):
+        config = get_config()
         scores = []
         for (release_type, release_type_slider) in self._release_type_sliders.items():
             scores.append((release_type, release_type_slider.value()))
@@ -264,7 +278,7 @@ class ReleasesOptionsPage(OptionsPage):
             source_list = [(c[0], gettext_countries(c[1])) for c in
                            source.items()]
         elif setting == "preferred_release_formats":
-            source_list = [(c[0], gettext_attr(c[1], "medium_format")) for c
+            source_list = [(c[0], pgettext_attributes("medium_format", c[1])) for c
                            in source.items()]
         else:
             source_list = [(c[0], _(c[1])) for c in source.items()]
@@ -272,11 +286,12 @@ class ReleasesOptionsPage(OptionsPage):
         def fcmp(x):
             return strxfrm(x[1])
         source_list.sort(key=fcmp)
+        config = get_config()
         saved_data = config.setting[setting]
         move = []
         for data, name in source_list:
             item = QtWidgets.QListWidgetItem(name)
-            item.setData(QtCore.Qt.UserRole, data)
+            item.setData(QtCore.Qt.ItemDataRole.UserRole, data)
             try:
                 i = saved_data.index(data)
                 move.append((i, item))
@@ -290,7 +305,8 @@ class ReleasesOptionsPage(OptionsPage):
         data = []
         for i in range(list1.count()):
             item = list1.item(i)
-            data.append(item.data(QtCore.Qt.UserRole))
+            data.append(item.data(QtCore.Qt.ItemDataRole.UserRole))
+        config = get_config()
         config.setting[setting] = data
 
 
